/****************************************************************************************/
/* TeslaCoilControl
 * Temperature control of both PWM fans via LM35 with an Arduino Nano CH340.
 * Switch off power supply in case of temperature alarm or over-current. 
 * LCD for better monitoring of the life status.
 *
 * Reference:
 * Origin Author:        Olaf Meier
 *
 * Modified by:          Olaf Meier
 *
 *
 * Hardware connection:  Arduino  -  Sensor
 *                       A0       -  LM35 Vout pin (5V Vout GND) for heat sink
 *                       A1       -  LM35 Vout pin (5V Vout GND) for coil
 *                       A2       -  Over-current measurement pin
 *                       3        -  PWM for heat sink fan
 *                       5        -  PWM for coil fan
 *                       4        -  Tesla coil generator power on/off
 *
 *                       Arduino  -  LCD
 *                       12       -  RS
 *                       11       -  E
 *                       10       -  D4
 *                        9       -  D5
 *                        8       -  D6
 *                        7       -  D7
 * 
 *
 * ToDo:
 *
 */
/****************************************************************************************/
/****************************************************************************************/
/***  Declaration of global constants and initialization of variables. Add includes.  ***/
/****************************************************************************************/
/***  Software release and date  ***/
const char* sketchname            =  "TeslaControlLCD";
const char* revision              =  " R.0.5";
const char* author                =  "Olaf Meier";
const char* date                  =  "2015/04/01";

/****************************************************************************************/
/***  Declare constants and variables for this sketch  ***/
const unsigned int BAUDRATE       =  9600;         // Baud rate for the serial monitor

// Include LCD library and initialize the library with the interface pins  ***/
#include <LiquidCrystal.h>
LiquidCrystal lcd(12, 11, 10, 9, 8, 7);            // RS, E, D4, D5, D6, D7

#define HEATSINKFAN                  3             // Pin to connect heat sink fan
#define COILFAN                      5             // Pin to connect coil fan
#define TESLAPOWER                   4             // Pin to switch on/off Tesla power
#define TEMPHEATSINKSENSOR           A0            // Connect LM35 for heat sink fan control
#define TEMPCOILSENSOR               A1            // Connect LM35 for Tesla coil fan
#define HEATSINKALARMLEVEL           60            // Switch off power at 60°C heat sink
#define COILALARMLEVEL               50            // Switch off power at 50°C coil temp
#define HYSTERESYS                   1             // 1°C hysteresys, time to cool down
const uint8_t measureCycles       =  20;           // Build average on some measure cycles
uint32_t tempHeatSink             =  0;            // Stores the actual heat sink temperature
uint32_t tempCoil                 =  0;            // Stores the actual Tesla coil temperature
#define CURRENTMEASUREMENTPIN        A2            // Pin to measure current
#define CURRENTALARMLEVEL            3500          // Switch of at 3500mA current
uint32_t current                  =  0;            // Stores the actual current
uint8_t alarmFlag                 =  0;            // Stores the kind of alarm
/****************************************************************************************/
/****************************************************************************************/
void setup() {

  Serial.begin(BAUDRATE);

  /***  Set up the LCD with number of columns and rows  ***/
  lcd.begin(16, 2);
  /***  Print a setup revision message to the LCD  ***/
  lcd.setCursor(0, 0);                             // Position cursor top left
  lcd.print(sketchname);
  lcd.setCursor(0, 1);
  lcd.print(author);
  lcd.print(revision);
  delay(1000);
  lcd.clear();                                     // Clear LCD 
  
  /***  Tesla coil generator power pin  ***/
  pinMode(TESLAPOWER, OUTPUT);
  digitalWrite(TESLAPOWER, HIGH);                  // Switch on power for Tesla coil
  
}                                                  // End of void setup()
/****************************************************************************************/
/****************************************************************************************/
void loop() {
  
  analogValues();                                  // Catch the actual temp and current

  /**************************************************************************************/
  /***  Print some debug values to the serial monitor  ***/ 
  Serial.print("Tesla power supply: ");            // Display the power on/off state
  Serial.println(digitalRead(TESLAPOWER));
  Serial.print("Heat sink temp.: ");               // Display heat sink temperature
  Serial.print(tempHeatSink);
  Serial.println(" C");
  Serial.print("Coil temp.:      ");
  Serial.print(tempCoil);                          // Display Tesla coil temperature
  Serial.println(" C");
  Serial.print("Current protection: ");
  Serial.print(current);
  Serial.println("mA");
  /**************************************************************************************/
  /***  Print some operation values to the LCD  ***/
  lcd.clear();                                     // Clear LCD 
  lcd.setCursor(0, 0);
  lcd.print("HS ");
  lcd.print(tempHeatSink);
  lcd.write(0xDF);
  lcd.print("C"); 
  lcd.setCursor(8, 0);
  lcd.print("TC ");
  lcd.print(tempCoil);
  lcd.write(0xDF);
  lcd.print("C"); 
  lcd.setCursor(0, 1);                             // 2nd row
  lcd.print(current);
  lcd.print("mA");
  lcd.setCursor(8, 1);
  if  (digitalRead(TESLAPOWER) == HIGH)
    lcd.print("Power On");
  else
    lcd.print("        ");
  lcd.setCursor(8, 1);                             // Position cursor
  if (alarmFlag == 1)
    lcd.print("OC Alarm");                         // Over current alarm
  if (alarmFlag == 2)            
    lcd.print("HS Alarm");                         // Temp. heat sink alarm 
  if (alarmFlag == 3)
    lcd.print("TC Alarm");                         // Temp. Tesla Coil alarm

  /**************************************************************************************/

  /***  Switch off Tesla coil power in case of any emergency  ***/
  emergencyPowerOff();                             // Emergency power off

  /***  Control speed of the fans  ***/
  fanControlPWM();

}                                                  // End of void loop()
/****************************************************************************************/
/****************************************************************************************/
/***  Measure and condition temperature via LM35 in full centigrades and current mA   ***/
void analogValues() {
  tempHeatSink = 0;                                // Reset former old values
  tempCoil = 0;
  current = 0;
  for (int i = 0; i < measureCycles; i++) {        // Start new measure cycle
    tempHeatSink += analogRead(TEMPHEATSINKSENSOR);
    tempCoil += analogRead(TEMPCOILSENSOR);
    current += analogRead(CURRENTMEASUREMENTPIN);
  }
  tempHeatSink /= measureCycles;                   // Build average
  tempCoil /= measureCycles;
  current /= measureCycles;
  /***  (5V reference * 100°C max. * measure value) / 10bit AD according 1024  ***/
  tempHeatSink = (tempHeatSink * 5 * 100) / 1023;  // LM35 conditioning to 10mV = 1°C
  tempCoil = (tempCoil * 5 * 100) / 1023;
  /***  Conditioning the measured current value in mA  ***/
  current = (current * (5000 / 1023))*10;          // 0,1R = 100mV with 1A
   
}                                                  // End of function void analogValues()
/****************************************************************************************/
/***  Conditioning the temperature values for the 8-bit PWM to control the fan speed  ***/
void fanControlPWM() {
  uint8_t heatSinkControl = constrain(tempHeatSink, 0, 50);
  uint8_t coilControl = constrain(tempCoil, 0, 50);// Constrain temp from 0°C to +50°C
  analogWrite(HEATSINKFAN, map(heatSinkControl, 0, 50, 0, 255));
  analogWrite(COILFAN, map(coilControl, 0, 50, 0, 255));// 8-bit PWM output is 0 to 255
}                                                  // End of function void fanControlPWM()
/****************************************************************************************/
/***  Switch off power for the Tesla coil generator in case of an emergency alarm  ***/
void emergencyPowerOff() {
  alarmFlag = 0;
  /***  Heat sink temperature alarm  ***/
  if (tempHeatSink > (HEATSINKALARMLEVEL + HYSTERESYS)) {
    digitalWrite(TESLAPOWER, LOW);
    alarmFlag = 2;
  }
  else if (tempHeatSink < (HEATSINKALARMLEVEL + HYSTERESYS)) {
    digitalWrite(TESLAPOWER, HIGH);
  }
  /***  Coil temperature alarm  ***/
  if (tempCoil > (COILALARMLEVEL + HYSTERESYS)) {
    digitalWrite(TESLAPOWER, LOW);
    alarmFlag = 3;
  }
  else if (tempCoil < (COILALARMLEVEL + HYSTERESYS)) {
    digitalWrite(TESLAPOWER, HIGH);
  }
  /***  Over-current protection alarm  ***/
  if (current > CURRENTALARMLEVEL) {
    digitalWrite(TESLAPOWER, LOW); 
    alarmFlag = 1;
  }
  else if (current < (CURRENTALARMLEVEL + HYSTERESYS)) {
    digitalWrite(TESLAPOWER, HIGH);
  } 
}                                                  // End of function emergencyPowerOff()
/****************************************************************************************/

/****************************************************************************************/
/****************************************************************************************/




